<?php

use Delo\DeloDebugInfo;
use Delo\DeloSettings;
use Delo\DeloLog;

/**
 * Class ControllerExtensionPaymentDelo
 * Расширение для связи с платежным шлюзом, корзина
 */
class ControllerExtensionPaymentDelo extends Controller
{
    /** @var ModelExtensionPaymentDelo */
    private $model;
    /** @var DeloSettings */
    private $settingsModel;
    /** @var DeloDebugInfo */
    private $debugInfo;
    /** @var array массив с настройками */
    private $settings;

    /**
     * ControllerExtensionPaymentDelo constructor.
     * @param Registry $registry
     * @throws Exception
     */
    public function __construct($registry)
    {
        parent::__construct($registry);
        // Подключение нужных классов
        $this->load->language('extension/payment/delo');
        $this->load->library('Delo/DeloErrors');
        $this->load->library('Delo/DeloLog');
        $this->load->library('Delo/DeloRequirements');
        $this->load->library('Delo/DeloSettings');
        $this->load->library('Delo/DeloDebugInfo');
        $this->settingsModel = $this->registry->get('DeloSettings');
        $this->settings = $this->settingsModel->getSettingsValue();
        $this->load->model('extension/payment/delo');
        $this->model = new ModelExtensionPaymentDelo($this->registry);
        $this->debugInfo = DeloDebugInfo::getInstance($registry);
    }

    /**
     * Возвращает кусок корзины с кнопкой подтверждения заказа
     * Просто делаем форму которая при submit-е перекидывает на метод confirm
     * @throws \Exception
     */
    public function index()
    {
        if ($this->settings['payment_page']) {
            $url = 'extension/payment/delo/confirm';
        } else {
            $url = 'extension/payment/delo/gotobank';
        }
        $data = [
            'action'         => $this->url->link($url, '', true),
            'order_id'       => $this->session->data['order_id'],
            'route'          => $url,
            'info'           => (!$this->settings['payment_page'] ? $this->settings['info'] : ''),
            'button_confirm' => $this->language->get('button_confirm'),
        ];
        return $this->load->view('extension/payment/delo_button', $data);
    }

    /**
     * Обработка подтверждение заказа
     * Перебрасывает на стартовую страницу платежного шлюза
     * @throws Exception
     */
    public function confirm()
    {
        // Сохраняем заказ, точнее он сохранен уже, надо ему присвоить первый статус, чтобы он отобразился в админке
        $this->model->setPendingStatusOrder($this->session->data['order_id'], $this->settings);
        $data = [
            'action'              => $this->url->link('extension/payment/delo/gotobank', '', true),
            'order_id'            => $this->session->data['order_id'],
            'route'               => 'extension/payment/delo/gotobank',
            'info'                => $this->settings['info'],
            'heading_title'       => $this->language->get('heading_title'),
            'button_payment_name' => $this->language->get('button_payment_name'),
            'column_left'         => $this->load->controller('common/column_left'),
            'column_right'        => $this->load->controller('common/column_right'),
            'content_top'         => $this->load->controller('common/content_top'),
            'content_bottom'      => $this->load->controller('common/content_bottom'),
            'header'              => $this->load->controller('common/header'),
            'footer'              => $this->load->controller('common/footer'),
            'breadcrumbs'         => [
                [
                    'text' => $this->language->get('text_home'),
                    'href' => $this->url->link('common/home')
                ],
                [
                    'text' => $this->language->get('text_cart'),
                    'href' => $this->url->link('checkout/cart')
                ],
                [
                    'text' => $this->language->get('checkout_title'),
                    'href' => $this->url->link('checkout/checkout')
                ],
                [
                    'text' => $this->language->get('heading_title'),
                    'href' => $this->url->link('extension/payment/delo/confirm', '', true)
                ],
            ]
        ];
        $this->response->setOutput($this->load->view('extension/payment/delo_confirm', $data));
    }

    /**
     * Обработка подтверждение заказа
     * Перебрасывает на стартовую страницу платежного шлюза
     * @throws Exception
     */
    public function gotobank()
    {
        if (!$this->settings['payment_page']) {
            // Сохраняем заказ, точнее он сохранен уже, надо ему присвоить первый статус, чтобы он отобразился в админке
            $this->model->setPendingStatusOrder($this->session->data['order_id'], $this->settings);
        }
        $args = [
            'merch_id'   => $this->settings['merch_id'],
            'o.order_id' => $this->session->data['order_id'],
            'back_url_s' => $this->settings['back_url_s'],
            'back_url_f' => $this->settings['back_url_f'],
            'lang'       => $this->settings['lang'],
        ];
        // Перебрасываем на стартовую страницу платежного шлюза
        $url = $this->settings['url'] . '?' . http_build_query($args);
        // Чистим сразу корзину, спорный момент, но пока так
        $this->cart->clear();
        $this->response->redirect($url);
    }

    /**
     * PaymentAvail Request
     * Проверка платежа
     * Этот метод ловит первый запрос от платежного шлюза
     */
    public function check()
    {
        $this->debugInfo->set('Request check');
        /**
         * Проверить trx_id
         */
        $debugId = $this->debugInfo->set('Проверить наличие trx_id... ');
        if (empty($this->request->get['trx_id'])) {
            $response = $this->model->checkResponseFailed($this->language->get('error_check_empty_trx_id'));
            $this->responseXml($response);
        }
        $this->debugInfo->setOk($debugId);
        $debugId = $this->debugInfo->set('Проверить что только разрешенные символы в trx_id... ');
        $trxId = $this->request->get['trx_id'];
        $error = $this->settingsModel->pcidValidate($trxId);
        if (!empty($error)) {
            $response = $this->model->checkResponseFailed($error);
            $this->responseXml($response);
        }
        $this->debugInfo->setOk($debugId);
        $debugId = $this->debugInfo->set('Проверить длину, должен быть 32 символа trx_id... ');
        $error = $this->settingsModel->pcidLengthValidate($trxId);
        if (!empty($error)) {
            $response = $this->model->checkResponseFailed($error);
            $this->responseXml($response);
        }
        $this->debugInfo->setOk($debugId);
        /**
         * Проверить merch_id
         */
        $debugId = $this->debugInfo->set('Проверить наличие merch_id... ');
        if (empty($this->request->get['merch_id'])) {
            $response = $this->model->checkResponseFailed($this->language->get('error_check_empty_merch_id'));
            $this->responseXml($response);
        }
        $this->debugInfo->setOk($debugId);
        $merchId = $this->request->get['merch_id'];
        /**
         * Проверить id заказа
         */
        $debugId = $this->debugInfo->set('Проверить наличие id заказа в запросе... ');
        if (empty($this->request->get['o_order_id'])) {
            $response = $this->model->checkResponseFailed($this->language->get('error_check_empty_order_id'));
            $this->responseXml($response);
        }
        $orderId = (int)$this->request->get['o_order_id'];
        if (empty($orderId)) {
            $response = $this->model->checkResponseFailed($this->language->get('error_check_empty_order_id'));
            $this->responseXml($response);
        }
        $this->debugInfo->setOk($debugId);
        $debugId = $this->debugInfo->set('Сравниваем merch_id с тем что в настройках... ');
        if ($this->settings['merch_id'] != $merchId) {
            $response = $this->model->checkResponseFailed('Failed merch_id');
            $this->responseXml($response);
        }
        $this->debugInfo->setOk($debugId);
        $ts = (empty($this->request->get['ts']) ? date('Ymd H:i:s') : $this->request->get['ts']);
        /**
         * Проверить существование заказа и что его стоимость не равна нулю
         */
        $debugId = $this->debugInfo->set('Ищем по id заказ в БД... ');
        $order = $this->model->getOrder($orderId);
        if (empty($order)) {
            $response = $this->model->checkResponseFailed($this->language->get('error_check_empty_order'));
            $this->model->logPaymentAvail($trxId, $orderId, $response, $this->request->get, $ts);
            $this->responseXml($response);
        }
        $this->debugInfo->setOk($debugId);
        $debugId = $this->debugInfo->set('Проверить сумму заказа > 0 ... ');
        $amount = $this->model->rublesToPennies($order['total']);
        if ($amount <= 0) {
            $response = $this->model->checkResponseFailed($this->language->get('error_check_amount_null'));
            $this->model->logPaymentAvail($trxId, $orderId, $response, $this->request->get, $ts);
            $this->responseXml($response);
        }
        $this->debugInfo->setOk($debugId);
        $debugId = $this->debugInfo->set('Все хорошо, готовим положительный ответ... ');
        // @todo: на основе заказа получить информацию о товарах
        // @todo: передавать информацию о покупателе доп. тегах - ФИО, телефон, email
        $response = $this->model->checkResponseSuccessful(
            'Проверка платежа заказа № ' . $orderId,
            'Оплата заказа № ' . $orderId,
            'Оплата заказа № ' . $orderId . ' ' . $this->config->get('config_name'),
            $amount
        );
        $this->model->logPaymentAvail($trxId, $orderId, $response, $this->request->get, $ts);
        $this->debugInfo->setOk($debugId);
        $this->responseXml($response);
    }

    /**
     * Фиксация платежа
     * @throws Exception
     */
    public function register()
    {
        $this->debugInfo->set('Request register');
        /**
         * Проверить trx_id
         */
        $debugId = $this->debugInfo->set('Проверить наличие trx_id... ');
        if (empty($this->request->get['trx_id'])) {
            $response = $this->model->registerResponseFailed($this->language->get('error_check_empty_trx_id'));
            $this->responseXml($response);
        }
        $this->debugInfo->setOk($debugId);
        $debugId = $this->debugInfo->set('Проверить что только разрешенные символы в trx_id... ');
        $trxId = $this->request->get['trx_id'];
        $error = $this->settingsModel->pcidValidate($trxId);
        if (!empty($error)) {
            $response = $this->model->registerResponseFailed($error);
            $this->responseXml($response);
        }
        $this->debugInfo->setOk($debugId);
        $debugId = $this->debugInfo->set('Проверить длину, должен быть 32 символа trx_id... ');
        $error = $this->settingsModel->pcidLengthValidate($trxId);
        if (!empty($error)) {
            $response = $this->model->registerResponseFailed($error);
            $this->responseXml($response);
        }
        $this->debugInfo->setOk($debugId);
        /**
         * Проверить merch_id
         */
        $debugId = $this->debugInfo->set('Проверить наличие merch_id... ');
        $merchId = '';
        if (empty($this->request->get['merch_id'])) {
            $response = $this->model->registerResponseFailed($this->language->get('error_check_empty_merch_id'));
            $this->responseXml($response);
        } else {
            $merchId = $this->request->get['merch_id'];
        }
        $this->debugInfo->setOk($debugId);
        /**
         * Проверить id заказа
         */
        $debugId = $this->debugInfo->set('Проверить наличие id заказа в запросе... ');
        if (empty($this->request->get['o_order_id'])) {
            $response = $this->model->registerResponseFailed($this->language->get('error_check_empty_order_id'));
            $this->responseXml($response);
        }
        $orderId = (int)$this->request->get['o_order_id'];
        if (empty($orderId)) {
            $response = $this->model->registerResponseFailed($this->language->get('error_check_empty_order_id'));
            $this->responseXml($response);
        }
        $this->debugInfo->setOk($debugId);
        /**
         * Проверить account_id
         */
        $debugId = $this->debugInfo->set('Проверить наличие account_id... ');
        $accountId = '';
        if (empty($this->request->get['account_id'])) {
            $response = $this->model->registerResponseFailed('Empty account id');
            $this->responseXml($response);
        } else {
            $accountId = $this->request->get['account_id'];
        }
        $this->debugInfo->setOk($debugId);
        $debugId = $this->debugInfo->set('Сравниваем merch_id с тем что в настройках... ');
        if ($this->settings['merch_id'] != $merchId) {
            $response = $this->model->registerResponseFailed('Failed merch_id');
            $this->responseXml($response);
        }
        $this->debugInfo->setOk($debugId);
        $debugId = $this->debugInfo->set('Сравниваем account_id с тем что в настройках... ');
        if ($this->settings['purchase_account_amount_id'] != $accountId) {
            $response = $this->model->registerResponseFailed('Failed account id');
            $this->responseXml($response);
        }
        $this->debugInfo->setOk($debugId);
        $ts = (empty($this->request->get['ts']) ? date('Ymd H:i:s') : $this->request->get['ts']);
        /**
         * Проверить существование заказа
         */
        $debugId = $this->debugInfo->set('Ищем по id заказ в БД... ');
        $order = $this->model->getOrder($orderId);
        if (empty($order)) {
            $response = $this->model->registerResponseFailed($this->language->get('error_check_empty_order'));
            $this->model->logRegisterPayment($trxId, $orderId, $response, $this->request->get, $ts);
            $this->responseXml($response);
        }
        $this->debugInfo->setOk($debugId);
        /**
         * Проверка существования result_code
         */
        $debugId = $this->debugInfo->set('Проверить наличие result_code... ');
        if (empty($this->request->get['result_code'])) {
            $response = $this->model->registerResponseFailed($this->language->get('error_check_empty_result_code'));
            $this->model->logRegisterPayment($trxId, $orderId, $response, $this->request->get, $ts);
            $this->model->setFailedStatusOrder($orderId, $this->settings);
            $this->responseXml($response);
        }
        $this->debugInfo->setOk($debugId);
        /**
         * Проверка amount
         */
        $debugId = $this->debugInfo->set('Проверить наличие amount... ');
        if (empty($this->request->get['amount'])) {
            $response = $this->model->registerResponseFailed($this->language->get('error_check_empty_amount'));
            $this->model->logRegisterPayment($trxId, $orderId, $response, $this->request->get, $ts);
            $this->model->setFailedStatusOrder($orderId, $this->settings);
            $this->responseXml($response);
        }
        $this->debugInfo->setOk($debugId);
        $debugId = $this->debugInfo->set('Проверить соответствует ли оплаченная сумма той, что в заказе... ');
        $amount = (int)$this->request->get['amount'];
        $amountOrder = $this->model->rublesToPennies($order['total']);
        if ($amount != $amountOrder) {
            $response = $this->model->registerResponseFailed($this->language->get('error_check_change_amount'));
            $this->model->logRegisterPayment($trxId, $orderId, $response, $this->request->get, $ts);
            $this->model->setFailedStatusOrder($orderId, $this->settings);
            $this->responseXml($response);
        }
        $this->debugInfo->setOk($debugId);
        /**
         * Цифровая подпись
         */
        $debugId = $this->debugInfo->set('Проверить наличие signature... ');
        if (!empty($this->request->get['signature'])) {
            $this->debugInfo->setOk($debugId);
            $this->debugInfo->set('Проверка цифровой подписи');
            if (!$this->model->checkSignature($this->request->get['signature'], $this->settings['check_signature'])) {
                $response = $this->model->registerResponseFailed($this->language->get('error_check_signature'));
                $this->model->logRegisterPayment($trxId, $orderId, $response, $this->request->get, $ts);
                $this->model->setFailedStatusOrder($orderId, $this->settings);
                $this->responseXml($response);
            }
        } else {
            $response = $this->model->registerResponseFailed($this->language->get('error_empty_digital_signature'));
            $this->model->logRegisterPayment($trxId, $orderId, $response, $this->request->get, $ts);
            $this->responseXml($response);
        }
        $result = (int)$this->request->get['result_code'];
        $this->debugInfo->set('result_code = "' . $result . '"');
        if ($result != 1) {
            $this->debugInfo->set('Смотрим расширенный код ошибки и отдаем успешный ответ...');
            if (!empty($this->request->get['ext_result_code'])) {
                $extResultCode = (string)((int)$this->request->get['ext_result_code']);
                $errorKey = 'error_register_' . $extResultCode;
                $error = $this->language->get($errorKey);
                if ($error == $errorKey) {
                    $error = $this->language->get('error_register_default');
                }
            } else {
                $error = $this->language->get('error_register_default');
            }
            $response = $this->model->registerResponseSuccessful($error);
            $this->model->logRegisterPayment($trxId, $orderId, $response, $this->request->get, $ts);
            $this->model->setFailedStatusOrder($orderId, $this->settings);
            $this->responseXml($response);
        }
        // Установка статуса заказу оплачен
        $debugId = $this->debugInfo->set('Установка статуса заказу оплачен... ');
        $this->model->setCompleteStatusOrder($orderId, $this->settings);
        $this->debugInfo->setOk($debugId);
        $this->debugInfo->set('Отдаем успешный ответ... ');
        $response = $this->model->registerResponseSuccessful('Заказ № ' . $orderId . ' - успех');
        $this->model->logRegisterPayment($trxId, $orderId, $response, $this->request->get, $ts);
        $this->responseXml($response);
    }

    /**
     * Выставляет заголовок и выплевывает xml
     * @param string $xml
     */
    private function responseXml($xml)
    {
        $this->response->addHeader('Content-type: application/xml');
        if (!empty($this->settings['debug_mode'])) {
            // Костыль конечно, но пох
            $xml = str_replace('</payment-avail-response>', $this->debugInfo->getXml() . '</payment-avail-response>', $xml);
            $xml = str_replace('</register-payment-response>', $this->debugInfo->getXml() . '</register-payment-response>', $xml);
        }
        $this->response->setOutput($xml);
        $this->response->output();
        exit;
    }
}
